﻿using System;
using System.Threading;
using System.Globalization;

/*
 * Copyright 2010 Arne Vajhøj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

using NUnit.Framework;

namespace Vajhoej.Record.Test
{
    [TestFixture]
    public class TestStructInfo
    {
        [Test]
        public void TestSimple()
        {
            try
            {
                StructInfo si = StructInfoCache.Analyze(typeof(Data));
                Assert.AreEqual(true, si.FixedLength, "fixed length");
                Assert.AreEqual(20, si.Length, "length");
            }
            catch (RecordException e)
            {
                Assert.Fail("Unexpected exception: " + e);
            }
        }
        [Test]
        public void TestAlign0()
        {
            try
            {
                StructInfo si = StructInfoCache.Analyze(typeof(AlignData0));
                Assert.AreEqual(true, si.FixedLength, "fixed length");
                Assert.AreEqual(15, si.Length, "length");
            }
            catch (RecordException e)
            {
                Assert.Fail("Unexpected exception: " + e);
            }
        }
        [Test]
        public void TestAlign1()
        {
            try
            {
                StructInfo si = StructInfoCache.Analyze(typeof(AlignData1));
                Assert.AreEqual(true, si.FixedLength, "fixed length");
                Assert.AreEqual(16, si.Length, "length");
            }
            catch (RecordException e)
            {
                Assert.Fail("Unexpected exception: " + e);
            }
        }
        [Test]
        public void TestAlign2()
        {
            try
            {
                StructInfo si = StructInfoCache.Analyze(typeof(AlignData2));
                Assert.AreEqual(true, si.FixedLength, "fixed length");
                Assert.AreEqual(32, si.Length, "length");
            }
            catch (RecordException e)
            {
                Assert.Fail("Unexpected exception: " + e);
            }
        }
        [Test]
        public void TestEndPad1()
        {
            try
            {
                StructInfo si = StructInfoCache.Analyze(typeof(EndPadData1));
                Assert.AreEqual(true, si.FixedLength, "fixed length");
                Assert.AreEqual(5, si.Length, "length");
            }
            catch (RecordException e)
            {
                Assert.Fail("Unexpected exception: " + e);
            }
        }
        [Test]
        public void TestEndPad2()
        {
            try
            {
                StructInfo si = StructInfoCache.Analyze(typeof(EndPadData2));
                Assert.AreEqual(true, si.FixedLength, "fixed length");
                Assert.AreEqual(8, si.Length, "length");
            }
            catch (RecordException e)
            {
                Assert.Fail("Unexpected exception: " + e);
            }
        }
        [Test]
        public void TestEndPad3()
        {
            try
            {
                StructInfo si = StructInfoCache.Analyze(typeof(EndPadData3));
                Assert.AreEqual(true, si.FixedLength, "fixed length");
                Assert.AreEqual(24, si.Length, "length");
            }
            catch (RecordException e)
            {
                Assert.Fail("Unexpected exception: " + e);
            }
        }
        [Test]
        public void TestSuperSub()
        {
            try
            {
                StructInfo sione = StructInfoCache.Analyze(typeof(SubDataOne));
                Assert.AreEqual(false, sione.FixedLength, "fixed length");
                Assert.AreEqual(16, sione.Length, "length");
                StructInfo sitwo = StructInfoCache.Analyze(typeof(SubDataTwo));
                Assert.AreEqual(false, sitwo.FixedLength, "fixed length");
                Assert.AreEqual(20, sitwo.Length, "length");
            }
            catch (RecordException e)
            {
                Assert.Fail("Unexpected exception: " + e);
            }
        }
        [Test]
        public void TestSuperSubPad()
        {
            try
            {
                StructInfo si = StructInfoCache.Analyze(typeof(SuperDataPad));
                Assert.AreEqual(true, si.FixedLength, "fixed length");
                Assert.AreEqual(20, si.Length, "length");
                StructInfo sione = StructInfoCache.Analyze(typeof(SubDataOnePad));
                Assert.AreEqual(true, sione.FixedLength, "fixed length");
                Assert.AreEqual(20, sione.Length, "length");
                StructInfo sitwo = StructInfoCache.Analyze(typeof(SubDataTwoPad));
                Assert.AreEqual(true, sitwo.FixedLength, "fixed length");
                Assert.AreEqual(20, sitwo.Length, "length");
            }
            catch (RecordException e)
            {
                Assert.Fail("Unexpected exception: " + e);
            }
        }
        [Test]
        public void TestMainField()
        {
            try
            {
                StructInfo si = StructInfoCache.Analyze(typeof(MainData));
                Assert.AreEqual(true, si.FixedLength, "fixed length");
                Assert.AreEqual(20, si.Length, "length");
            }
            catch (RecordException e)
            {
                Assert.Fail("Unexpected exception: " + e);
            }
        }
    }
}
